"""Init file for molara package."""

from __future__ import annotations

__copyright__ = "Copyright 2024, Molara"
__version__ = "0.0.2"
"""This variable is used as a single source of truth for the version.

All other parts that need the version such as pyproject.toml and the sphinx
documentation should import it from here.
"""
